@@ -3,22 +3,22 @@ try:
 except ImportError:
     from builtins import all as _all, any as _any
 from operator import itemgetter, methodcaller, attrgetter
-from collections import Mapping, Set, Iterable, Iterator, defaultdict
 from itertools import chain, tee
+from collections import defaultdict
 
-from .cross import basestring, xrange, izip, map, filter, imap, PY2
+from .compat import basestring, range, zip, map, filter, PY2, Mapping, Set, Iterable, Iterator
 from .primitives import EMPTY
 from .funcs import partial, compose
 from .funcmakers import make_func, make_pred
-from .seqs import take, ximap, ifilter
+from .seqs import take, xmap, filter as xfilter
 
 
 __all__ = ['empty', 'iteritems', 'itervalues',
            'join', 'merge', 'join_with', 'merge_with',
            'walk', 'walk_keys', 'walk_values', 'select', 'select_keys', 'select_values', 'compact',
            'is_distinct', 'all', 'any', 'none', 'one', 'some',
-           'zipdict', 'flip', 'project', 'omit', 'izip_values', 'izip_dicts',
-           'where', 'pluck', 'pluck_attr', 'invoke', 'iwhere', 'ipluck', 'ipluck_attr', 'iinvoke',
+           'zipdict', 'flip', 'project', 'omit', 'zip_values', 'zip_dicts',
+           'where', 'pluck', 'pluck_attr', 'invoke', 'lwhere', 'lpluck', 'lpluck_attr', 'linvoke',
            'get_in', 'set_in', 'update_in']
 
 
@@ -31,18 +31,20 @@ FACTORY_REPLACE = {
 }
 
 def _factory(coll, mapper=None):
+    coll_type = type(coll)
     # Hack for defaultdicts overriden constructor
     if isinstance(coll, defaultdict):
-        item_factory = compose(mapper, coll.default_factory) if mapper else coll.default_factory
+        item_factory = compose(mapper, coll.default_factory) if mapper and coll.default_factory \
+                       else coll.default_factory
         return partial(defaultdict, item_factory)
     elif isinstance(coll, Iterator):
         return iter
     elif isinstance(coll, basestring):
-        return ''.join
-    elif type(coll) in FACTORY_REPLACE:
-        return FACTORY_REPLACE[type(coll)]
+        return coll_type().join
+    elif coll_type in FACTORY_REPLACE:
+        return FACTORY_REPLACE[coll_type]
     else:
-        return coll.__class__
+        return coll_type
 
 def empty(coll):
     """Creates an empty collection of the same type."""
@@ -86,7 +88,7 @@ def join(colls):
         return result
     elif isinstance(dest, Set):
         return dest.union(*it)
-    elif isinstance(dest, (Iterator, xrange)):
+    elif isinstance(dest, (Iterator, range)):
         return chain.from_iterable(colls)
     elif isinstance(dest, Iterable):
         # NOTE: this could be reduce(concat, ...),
@@ -134,7 +136,7 @@ def merge_with(f, *dicts):
 def walk(f, coll):
     """Walks the collection transforming its elements with f.
        Same as map, but preserves coll type."""
-    return _factory(coll)(ximap(f, iteritems(coll)))
+    return _factory(coll)(xmap(f, iteritems(coll)))
 
 def walk_keys(f, coll):
     """Walks keys of the collection, mapping them with f."""
@@ -154,13 +156,13 @@ def walk_values(f, coll):
         k, v = pair
         return k, f(v)
 
-    return _factory(coll, mapper=f)(ximap(pair_f, iteritems(coll)))
+    return _factory(coll, mapper=f)(xmap(pair_f, iteritems(coll)))
 
 # TODO: prewalk, postwalk and friends
 
 def select(pred, coll):
     """Same as filter but preserves coll type."""
-    return _factory(coll)(ifilter(pred, iteritems(coll)))
+    return _factory(coll)(xfilter(pred, iteritems(coll)))
 
 def select_keys(pred, coll):
     """Select part of the collection with keys passing pred."""
@@ -188,20 +190,20 @@ def is_distinct(coll, key=EMPTY):
     if key is EMPTY:
         return len(coll) == len(set(coll))
     else:
-        return len(coll) == len(set(ximap(key, coll)))
+        return len(coll) == len(set(xmap(key, coll)))
 
 
 def all(pred, seq=EMPTY):
     """Checks if all items in seq pass pred (or are truthy)."""
     if seq is EMPTY:
         return _all(pred)
-    return _all(ximap(pred, seq))
+    return _all(xmap(pred, seq))
 
 def any(pred, seq=EMPTY):
     """Checks if any item in seq passes pred (or is truthy)."""
     if seq is EMPTY:
         return _any(pred)
-    return _any(ximap(pred, seq))
+    return _any(xmap(pred, seq))
 
 def none(pred, seq=EMPTY):
     """"Checks if none of the items in seq pass pred (or are truthy)."""
@@ -211,25 +213,25 @@ def one(pred, seq=EMPTY):
     """Checks whether exactly one item in seq passes pred (or is truthy)."""
     if seq is EMPTY:
         return one(bool, pred)
-    return len(take(2, ifilter(pred, seq))) == 1
+    return len(take(2, xfilter(pred, seq))) == 1
 
 # Not same as in clojure! returns value found not pred(value)
 def some(pred, seq=EMPTY):
     """Finds first item in seq passing pred or first that is truthy."""
     if seq is EMPTY:
         return some(bool, pred)
-    return next(ifilter(pred, seq), None)
+    return next(xfilter(pred, seq), None)
 
 # TODO: a variant of some that returns mapped value,
-#       one can use some(imap(f, seq)) or first(ikeep(f, seq)) for now.
+#       one can use some(map(f, seq)) or first(keep(f, seq)) for now.
 
 # TODO: vector comparison tests - ascending, descending and such
 # def chain_test(compare, seq):
-#     return all(compare, izip(seq, rest(seq))
+#     return all(compare, zip(seq, rest(seq))
 
 def zipdict(keys, vals):
     """Creates a dict with keys mapped to the corresponding vals."""
-    return dict(izip(keys, vals))
+    return dict(zip(keys, vals))
 
 def flip(mapping):
     """Flip passed dict or collection of pairs swapping its keys and values."""
@@ -246,7 +248,7 @@ def omit(mapping, keys):
     """Removes given keys from mapping."""
     return _factory(mapping)((k, v) for k, v in iteritems(mapping) if k not in keys)
 
-def izip_values(*dicts):
+def zip_values(*dicts):
     """Yields tuples of corresponding values of several dicts."""
     if len(dicts) < 1:
         raise TypeError('izip_values expects at least one argument')
@@ -254,7 +256,7 @@ def izip_values(*dicts):
     for key in keys:
         yield tuple(d[key] for d in dicts)
 
-def izip_dicts(*dicts):
+def zip_dicts(*dicts):
     """Yields tuples like (key, val1, val2, ...)
        for each common key in all given dicts."""
     if len(dicts) < 1:
@@ -293,39 +295,41 @@ def update_in(coll, path, update, default=None):
         return copy
 
 
-def where(mappings, **cond):
+def lwhere(mappings, **cond):
     """Selects mappings containing all pairs in cond."""
-    match = lambda m: all(m[k] == v for k, v in cond.items())
+    return list(where(mappings, **cond))
+
+def lpluck(key, mappings):
+    """Lists values for key in each mapping."""
+    return list(pluck(key, mappings))
+
+def lpluck_attr(attr, objects):
+    """Lists values of given attribute of each object."""
+    return list(pluck_attr(attr, objects))
+
+def linvoke(objects, name, *args, **kwargs):
+    """Makes a list of results of the obj.name(*args, **kwargs)
+       for each object in objects."""
+    return list(invoke(objects, name, *args, **kwargs))
+
+
+# Iterator versions for python 3 interface
+
+def where(mappings, **cond):
+    """Iterates over mappings containing all pairs in cond."""
+    items = cond.items()
+    match = lambda m: all(k in m and m[k] == v for k, v in items)
     return filter(match, mappings)
 
 def pluck(key, mappings):
-    """Lists values for key in each mapping."""
+    """Iterates over values for key in mappings."""
     return map(itemgetter(key), mappings)
 
 def pluck_attr(attr, objects):
-    """Lists values of given attribute of each object."""
+    """Iterates over values of given attribute of given objects."""
     return map(attrgetter(attr), objects)
 
 def invoke(objects, name, *args, **kwargs):
-    """Makes a list of results of the obj.name(*args, **kwargs)
-       for each object in objects."""
-    return map(methodcaller(name, *args, **kwargs), objects)
-
-# Iterator versions for python 3 interface
-def iwhere(mappings, **cond):
-    """Iterates over mappings containing all pairs in cond."""
-    match = lambda m: all(m[k] == v for k, v in cond.items())
-    return ifilter(match, mappings)
-
-def ipluck(key, mappings):
-    """Iterates over values for key in mappings."""
-    return imap(itemgetter(key), mappings)
-
-def ipluck_attr(attr, objects):
-    """Iterates over values of given attribute of given objects."""
-    return imap(attrgetter(attr), objects)
-
-def iinvoke(objects, name, *args, **kwargs):
     """Yields results of the obj.name(*args, **kwargs)
        for each object in objects."""
-    return imap(methodcaller(name, *args, **kwargs), objects)
+    return map(methodcaller(name, *args, **kwargs), objects)
